<?php

namespace Drupal\entity_reference_preview\Plugin\PreviewDetector;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\State\StateInterface;
use Drupal\entity_reference_preview\PreviewDetectorPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Plugin implementation of the preview_detector.
 *
 * @PreviewDetector(
 *   id = "state",
 *   label = @Translation("State"),
 *   description = @Translation("Detects the preview state on a value in the State API.")
 * )
 */
final class StatePreviewDetector extends PreviewDetectorPluginBase implements ContainerFactoryPluginInterface {

  const STATE_KEY = 'entity_reference_preview.status';
  const STARTED = 'previewing';
  const STOPPED = 'stopped';

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  private $state;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, StateInterface $state) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('state')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isPreviewing(Request $request): bool {
    return $this->state->get(static::STATE_KEY) === static::STARTED;
  }

  /**
   * Starts previewing.
   */
  public function start(): void {
    $this->state->set(static::STATE_KEY, static::STARTED);
  }

  /**
   * Stops previewing.
   */
  public function stop() {
    $this->state->set(static::STATE_KEY, static::STOPPED);
  }

}
